package Outils;

import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.*;

/** Classe pour la gestion des dates.

<br><br>
<font size=+1><B>Un peu d'histoire.</B></font><br>
Jules C&eacute;sar mit en vigueur, en 45 avant J.-C., le calendrier connu actuellement sous le terme de calendrier julien. Il mettait ainsi un peu d'ordre dans les calendriers pr&eacute;c&eacute;dents qui &eacute;taient tr&egrave;s irr&eacute;guliers.<br>
Le calendrier julien fut cependant modifi&eacute; en 8 avant J.-C.. En effet, des erreurs avaient amen&eacute; un d&eacute;calage par rapport aux saisons. De 45 avant J.-C. &agrave; 8 avant J.C., les mois comprenaient le nombre de jours suivant:<br>
<CENTER><B><TABLE BORDER=0 WIDTH=50%>
<TR><TD>janvier: 31</TD><TD>f&eacute;vrier 29 ou 30</TD><TD> mars: 31 </TD></TR>
<TR><TD>avril: 30</TD><TD> mai: 31</TD><TD> juin: 30 </TD></TR>
<TR><TD>juillet: 31</TD><TD> aot: 30</TD><TD> septembre: 31 </TD></TR>
<TR><TD>octobre: 30</TD><TD> novembre: 31</TD><TD> d&eacute;cembre: 30 </TD></TR>
</TABLE></B></CENTER>
De plus, les ann&eacute;es bissextiles de 45 avant J.-C. &agrave; 8 avant J.C. furent erron&eacute;ment calcul&eacute;es: 45, 42, 39, 36, 33, 30, 27, 24, 21, 18, 15, 12, 9 et ensuite, plus aucune jusqu'en 8 apr&egrave;s J.-C.<br>
En l'an 1582, le pape Gr&eacute;goire XIII d&eacute;cide de r&eacute;former le calendrier julien qui &eacute;tait d'application. A Rome, comme en France, le 5 octobre 1582 (2299161 &egrave;me jour julien) devint le 15 octobre. En Grande Bretagne, il fallut attendre l'ann&eacute;e 1752 pour que le 3 septembre (2361222 &egrave;me jour julien) devienne le 14 septembre.<br>
Ce changement &eacute;tait rendu n&eacute;cessaire par le d&eacute;calage d &agrave; la longueur d'une ann&eacute;e julienne: 365,25 jours alors qu'en r&eacute;alit&eacute;, une ann&eacute;e (dur&eacute;e entre 2 passages cons&eacute;cutifs du soleil au tropique) comporte 365,24220 jours.<br>
Malgr&eacute; l'adoption de ce nouveau calendrier (calendrier gr&eacute;gorien) o une ann&eacute;e comporte 365,2425 jours, le calcul du nombre de jours selon le calendrier julien reste une convention astronomique pour le calcul des dur&eacute;es selon un temps universel. Le passage au calendrier gr&eacute;gorien ne se fit d'ailleurs pas de mani&egrave;re simultan&eacute;e dans tous les pays: en 1752 au Royaume-Uni, en 1918 en URSS et en 1928 en Turquie.<br>
Il faut remarquer que le premier janvier n'a pas toujours &eacute;t&eacute; le premier jour de l'ann&eacute;e. En particulier, au Royaume-Uni, l'ann&eacute;e a commenc&eacute; le 25 mars jusqu'en 1752. D'une mani&egrave;re assez surprenante, l'ann&eacute;e se terminait, elle, le 31 d&eacute;cembre laissant pendants les jours entre ces 2 dates.<br>
Autre particularit&eacute; remarquable: pendant longtemps, le jour suppl&eacute;mentaire que comptait l'ann&eacute;e bissextile &eacute;tait rajout&eacute; le 24 f&eacute;vrier (ou le 25 de 45 &agrave; 8 avant J.-C.) qui &eacute;tait compt&eacute; 2 fois.<br>
 */
public class DATE {
	private int m_Annee=0;
	private int m_Mois=0;
	private int m_Jour=0;
	private int m_Heure=0;
	private int m_Minute=0;
	private int GMT=0;

/** Cr&eacute;ation d'une nouvelle instance de la classe.
 * @param d date &agrave; ins&eacute;rer au format JJ/MM/AAAA
 * @param h heure &agrave; ins&eacute;rer au format HH:MM
 */	
	public DATE (String d, String h) throws DATEException{
		String delim = "/";
		StringTokenizer st;
		int limite;
		if (formatDate(d)){
			st = new StringTokenizer(d,delim);
			limite = st.countTokens();
			m_Jour=(new Integer(st.nextToken())).intValue();
			m_Mois=(new Integer(st.nextToken())).intValue();
			m_Annee=(new Integer(st.nextToken())).intValue();
		}
		else throw new DATEException("DATE <init> : format de la date invalide (JJ/MM/AAAA)");
		
		delim=":";
		if (formatHeure(h)){
			st = new StringTokenizer(h,delim);
			limite = st.countTokens();
			m_Heure=(new Integer(st.nextToken())).intValue();
			m_Minute=(new Integer(st.nextToken())).intValue();
		}
		else throw new DATEException("DATE <init> : format de l'heure invalide (HH:MM)");
	}

/** Cr&eacute;e une nouvelle instance de la classe.
 */	
	public DATE () throws DATEException{
		Calendar calendar = new GregorianCalendar();

		m_Jour=calendar.get(Calendar.DAY_OF_MONTH);
		m_Mois=calendar.get(Calendar.MONTH)+1;
		m_Annee=calendar.get(Calendar.YEAR);
		if(calendar.get(Calendar.AM_PM)>0) m_Heure=calendar.get(Calendar.HOUR)+12;
		else m_Heure=calendar.get(Calendar.HOUR);
		m_Minute=calendar.get(Calendar.MINUTE);
	}

/** Cr&eacute;ation d'une nouvelle instance de la classe.
 * @param j le jour.
 * @param m le mois.
 * @param a l'ann&eacute;e.
 */	
	public DATE (int j, int m, int a) throws DATEException{
		Calendar calendar = new GregorianCalendar();

		m_Jour=j;
		m_Mois=m;
		m_Annee=a;
		if(calendar.get(Calendar.AM_PM)>0) m_Heure=calendar.get(Calendar.HOUR)+12;
		else m_Heure=0;
		m_Minute=0;
	}
	
		
/** Test si la date de l'objet est strictement inf&eacute;rieure &agrave; une autre date.
 * @param d la date &agrave; comparer
 * @return Vrai si l'instance est inf&eacute;rieure &agrave; la date pass&eacute;e en param&egrave;tre.
 * @see <a href="DATE.html#sup(DATE)">sup</a>
 * @see <a href="DATE.html#egal(DATE)">egal</a>
 */	
	public boolean inf(DATE d){
		if (m_Annee<d.getAnnee()) return true;
		else if (m_Annee>d.getAnnee()) return false;
		else if (m_Mois<d.getMois()) return true;
		else if (m_Mois>d.getMois()) return false;
		else if (m_Jour<d.getJour()) return true;
		else if (m_Jour>d.getJour()) return false;
		else if (m_Heure<d.getHeure()) return true;
		else if (m_Heure>d.getHeure()) return false;
		else if (m_Minute<d.getMinute()) return true;
		else if (m_Minute>d.getMinute()) return false;
		else return false;
	}
	
/** Test si la date de l'objet est strictement sup&eacute;rieure &agrave; une autre date.
 * @param d la date &agrave; comparer
 * @return Vrai si l'instance est sup&eacute;rieure &agrave; la date pass&eacute;e en param&egrave;tre.
 * @see <a href="DATE.html#inf(DATE)">inf</a>
 * @see <a href="DATE.html#egal(DATE)">egal</a>
 */	
	public boolean sup(DATE d){
		if (m_Annee>d.getAnnee()) return true;
		else if (m_Annee<d.getAnnee()) return false;
		else if (m_Mois>d.getMois()) return true;
		else if (m_Mois<d.getMois()) return false;
		else if (m_Jour>d.getJour()) return true;
		else if (m_Jour<d.getJour()) return false;
		else if (m_Heure>d.getHeure()) return true;
		else if (m_Heure<d.getHeure()) return false;
		else if (m_Minute>d.getMinute()) return true;
		else if (m_Minute<d.getMinute()) return false;
		else return false;
	}

/** Test si la date de l'objet est &eacute;gale &agrave; une autre date.
 * @param d la date &agrave; comparer
 * @return Vrai si l'instance est &eacute;gale &agrave; la date pass&eacute;e en param&egrave;tre.
 * @see <a href="DATE.html#inf(DATE)">inf</a>
 * @see <a href="DATE.html#sup(DATE)">sup</a>
 */	
	public boolean egal(DATE d){
		return (m_Annee==d.getAnnee() && m_Mois==d.getMois() && m_Jour==d.getJour() && m_Heure==d.getHeure() && m_Minute<d.getMinute());
	}
	
/** R&eacute;cup&eacute;ration de l'ann&eacute;e.
 * @return la valeur d&eacute;cimale de l'ann&eacute;e.
 */
	public int getAnnee(){
		return m_Annee;
	}

/** R&eacute;cup&eacute;ration du mois.
 * @return la valeur d&eacute;cimale du mois.
 */
	public int getMois(){
		return m_Mois;
	}

/** R&eacute;cup&eacute;ration du jour.
 * @return la valeur d&eacute;cimale du jour.
 */
	public int getJour(){
		return m_Jour;
	}

/** R&eacute;cup&eacute;ration de l'heure.
 * @return la valeur d&eacute;cimale de l'heure.
 */
	public int getHeure(){
		return m_Heure;
	}

/** R&eacute;cup&eacute;ration des minutes.
 * @return la valeur d&eacute;cimale des minutes.
 */
	public int getMinute(){
		return m_Minute;
	}

/** Change l'horaire (heures + minutes) d'une date.
 * @param h heures.
 * @param m minutes.
 * @return la nouvelle date.
 */
	public DATE setHoraire(int h,int m){
		m_Heure = h;
		m_Minute = m;
		return this;
	}

/** Convertit la classe DATE en chaine de caract&egrave;re de la forme JJ/MM/AAAA HH:MM
 * @return la chaine de caract&egrave;re.
 */
	public String toString(){
		String tmp;
		String retour="";
		tmp=""+m_Jour;
		if (tmp.length()==1) retour+=("0"+m_Jour+"/");
		else retour+=(m_Jour+"/");
		tmp=""+m_Mois;
		if (tmp.length()==1) retour+=("0"+m_Mois+"/");
		else retour+=(m_Mois+"/");
		tmp=""+m_Annee;
		if (tmp.length()==1) retour+=("000"+m_Annee+" ");
		else if (tmp.length()==2) retour+=("00"+m_Annee+" ");
		else if (tmp.length()==3) retour+=("0"+m_Annee+" ");
		else retour+=m_Annee+" ";
		
		tmp=""+m_Heure;
		if (tmp.length()==1) retour+=("0"+m_Heure);
		else retour+=(m_Heure+":");
		tmp=""+m_Minute;
		if (tmp.length()==1) retour+=("0"+m_Minute);
		else retour+=(m_Minute);
		
		return retour;
	}
	
/** V&eacute;rification du format de la date.
 * @param s la date au format JJ/MM/AAAA.
 * @return Vrai si le format est respect&eacute; et si la date est coh&eacute;rente ( par exemple : il n'existe pas de 31 f&eacute;vrier).
 */
	public static boolean formatDate(String s){
		String delim = "/";
		StringTokenizer st;
		int limite;
		int a;
		int m;
		int j;
		String jour;
		String mois;
		String annee;
		
		st = new StringTokenizer(s,delim);
		limite = st.countTokens();
		
		if (limite==3){
			jour = st.nextToken();
			mois = st.nextToken();
			annee = st.nextToken();
			try{
				j=(new Integer(jour)).intValue();
				m=(new Integer(mois)).intValue();
				a=(new Integer(annee)).intValue();
			}
			catch(NumberFormatException e){
				return false;
			}
			
			if (annee.length()<4) return false;
			if (j<=0) return false;

			switch (m){
				case 1 : if (j>31) return false;break;
				case 3 : if (j>31) return false;break;
				case 5 : if (j>31) return false;break;
				case 7 : if (j>31) return false;break;
				case 8 : if (j>31) return false;break;
				case 10 : if (j>31) return false;break;
				case 12 : if (j>31) return false;break;
				case 2 :
					if (bissextile(a)){
						if (j >29) return false;
					}
					else{
						if (j >28) return false;
					}
					break;
				case 4 : if (j>30) return false;break;
				case 6 : if (j>30) return false;break;
				case 9 : if (j>30) return false;break;
				case 11 : if (j>30) return false;break;
				default : return false;
			}
		return true;
		}
		else return false;
	}
	
/** V&eacute;rification du format de l'heure.
 * @param s l'heure au format HH:MM.
 * @return Vrai si le format est respect&eacute; et si l'heure est coh&eacute;rente (par exemple : il n'existe pas de 24:54).
 */
	public static boolean formatHeure(String s){
		String delim = ":";
		StringTokenizer st;
		int limite;
		int tmp;
		
		st = new StringTokenizer(s,delim);
		limite = st.countTokens();
		if (limite==2){
			try{
				tmp=(new Integer(st.nextToken())).intValue();
				if (tmp>23 || tmp<0) return false;
				tmp=(new Integer(st.nextToken())).intValue();
				if (tmp>59 || tmp<0) return false;
			}
			catch (NumberFormatException e){
				return false;
			}
		}
		else return false;
		return true;
	}

/** Calcul du nom des jours de la semaine &agrave; partir de 1582 prenant pour r&eacute;f&eacute;rence le mercredi 01/01/1997.

<br><BR><b>Algorithme pour d&eacute;terminer le jour de la semaine correspondant &agrave; une date (j/m/a) :</b><br>
L'algorithme n'est valable que pour les dates post&eacute;rieures &agrave; 1582.<br>
................................<br>

 * @param j la valeur d&eacute;cimale du jour.
 * @param j la valeur d&eacute;cimale du mois.
 * @param j la valeur d&eacute;cimale de l'ann&eacute;e.
 * @return 0 : LUNDI, 1 : MARDI, 2 : MERCREDI, 3 : JEUDI, 4 : VENDREDI, 5 : SAMEDI, 6 : DIMANCHE.
 * @see <a href="DATE.html#jourSemaine1582(int, int, int)">jourSemaine1582</a>
 */
	public static int jourSemaine1997 (int j, int m, int a)throws DATEException{
		int rep1 = 0;
		int rep2 = 0;
		
		if(a>=1997){
			for(int i=1997;i<a;i++){
				if(bissextile(i)) rep1+=366;
				else rep1+=365;
			}
			for(int k=1;k<m;k++){
				try{
					rep2+=jourMois(k,a);
				}
				catch (DATEException e){
					throw e;
				}
			}
			return (rep1+rep2+j+1)%7;
		}
		else{
			for(int z=1996;z>a;z--){
				if(bissextile(z)) rep1+=366;
				else rep1+=365;
			}
			for(int y=12;y>=m;y--){
				try{
					rep2+=jourMois(y,a);
				}
				catch (DATEException e){
					throw e;
				}
			}
			return (7-((rep1+rep2-j-1)%7))%7;
		}
	}
	
/** Calcul du nom des jours de la semaine &agrave; partir de 1582.

<br><b>Algorithme pour d&eacute;terminer le jour de la semaine correspondant &agrave; une date (j/m/a) :</b><br>
L'algorithme n'est valable que pour les dates post&eacute;rieures &agrave; 1582.<br>
   <code>Pour janvier et f&eacute;vrier, il faut augmenter m de 12 et diminuer a de 1.<br>
   Calculer s qui vaut la partie enti&egrave;re de a divis&eacute; par 100<br>
            JD = 1720996,5 - s + s \ 4 + [365,25*a] + [30,6001*(m+1)] + j<br>
            JD = JD - [JD/7]*7<br>
            JS = [JD] MOD 7<br>
   si JS =  0, le jour est mardi,<br>
   si JS =  1, le jour est mercredi,<br>
   ....<br>
   si JS =  7, le jour est lundi.</code><br>
 * @param j la valeur d&eacute;cimale du jour.
 * @param j la valeur d&eacute;cimale du mois.
 * @param j la valeur d&eacute;cimale de l'ann&eacute;e.
 * @return 0 : LUNDI, 1 : MARDI, 2 : MERCREDI, 3 : JEUDI, 4 : VENDREDI, 5 : SAMEDI, 6 : DIMANCHE.
 * @see <a href="DATE.html#jourSemaine1997(int, int, int)">jourSemaine1997</a>
 */
	public static int jourSemaine1582 (int jour, int mois, int annee){
		int s=0; //partie entire de a divise par 100
		double JD=0;
		double tmp;
		int JS=0; // jour de la semaine
		int m=mois;
		int j=jour;
		int a=annee;
		
		if (m<3) {
			m+=12;
			a--;
		}
		s = a/100;
		JD=((1720996.5) - s + (int)(s/4) + (int)(365.25*a) + (int)(30.6001*(m+1)) + j);
		JD=JD-(int)(JD/7)*7;
		JS =(int) ((JD+1)%7);
		return JS;
	}

/** Test des ann&eacute;es bissextiles.

<br><BR><B> Algorithme pour d&eacute;terminer si l'ann&eacute;e A est bissextile. </B><br><br>
<code>
   Si A n'est pas divisible par 4, l'ann&eacute;e n'est pas bissextile.<br>
   Si A est divisible par 4, l'ann&eacute;e est bissextile sauf si A est divisible par 100 et pas par 400.</code><br>
 * @param a la valeur d&eacute;cimale de l'ann&eacute;e.
 * @return Vrai si l'ann&eacute;e est bissextile.
 */
	public static boolean bissextile (int a){
		return (((a % 400) == 0)||(((a % 100) != 0)&&((a % 4) == 0)));
	}


/** Calcul du nombre de jours dans un mois.
 * @param m la valeur d&eacute;cimale du mois.
 * @param a la valeur d&eacute;cimale de l'ann&eacute;e.
 * @return le nombre de jours dans le mois.
 */
	public static int jourMois(int m, int a) throws DATEException{
		switch (m){
			case 1 : return 31;
			case 3 : return 31;
			case 5 : return 31;
			case 7 : return 31;
			case 8 : return 31;
			case 10 : return 31;
			case 12 : return 31;
			case 2 :
				if (bissextile(a)) return 29;
				else return 28;
			case 4 : return 30;
			case 6 : return 30;
			case 9 : return 30;
			case 11 : return 30;
			default : throw new DATEException("DATE.jourMois valeur du mois invalide");
		}
	}	
	
/** Calcul du nombre de jours dans le mois de la classe instanci&eacute;e.
 * @return le nombre de jours dans le mois.
 */
	public int jourMois(){
		try {
			return jourMois(m_Mois,m_Annee);
		}
		catch (DATEException e){
			return -1;
		}
	}
	
/** G&eacute;n&egrave;ration de la premi&egrave;re lettre d'un jour de l'ann&eacute;e.
 * @param jour la valeur d&eacute;cimale du jour.
 * @param mois la valeur d&eacute;cimale du mois.
 * @param annee la valeur d&eacute;cimale de l'annee.
 * @return la premi&egrave;re lettre du jour.
 */
	public String jourToString(int jour, int mois, int annee){
		return jourToString(jourSemaine1582 (jour,mois,annee));
	}

/** Permet de r&eacute;cup&eacute;rer la valeur num&eacute;rique du jour de la semaine de l'instance.
 * @return 	la valeur num&eacute;rique du jour de la semaine de l'instance.
 */
	public int jourSemaine(){
		return jourSemaine1582(m_Jour,m_Mois,m_Annee);
	}
	
/** Permet de r&eacute;cup&eacute;rer la valeur num&eacute;rique du jour de la semaine de la date pass&eacute;e en argument.
 * @param j le jour.
 * @param m le mois.
 * @param a l'ann&eacute;e.
 * @return 	la valeur num&eacute;rique du jour de la semaine.
 */
	public int jourSemaine(int j, int m, int a){
		return jourSemaine1582(j,m,a);
	}
	
/** Genere la premi&egrave;re lettre du jour.
 * @param j la valeur d&eacute;cimale du jour.
 * @return la premi&egrave;re lettre du jour.
 */
	public String jourToString(int j){
		switch (j){
			case 0 : return "L";
			case 1 : return "M";
			case 2 : return "M";
			case 3 : return "J";
			case 4 : return "V";
			case 5 : return "S";
			case 6 : return "D";
			default : return "";
		}
	}
	
/** G&eacute;n&egrave;ration le nom du mois.
 * @param m la valeur d&eacute;cimale du mois.
 * @return le nom du mois.
 */
	public String moisToString(int m){
		switch (m){
			case 1 : return "Janvier";
			case 2 : return "Fevrier";
			case 3 : return "Mars";
			case 4 : return "Avril";
			case 5 : return "Mai";
			case 6 : return "Juin";
			case 7 : return "Juillet";
			case 8 : return "Aout";
			case 9 : return "Septembre";
			case 10 : return "Octobre";
			case 11 : return "Novembre";
			case 12 : return "Decembre";
			default : return "";
		}
	}
	
/** Calcul le num&eacute;ro de semaine d'un jour de l'ann&eacute;e.
 * @return la valeur de la semaine.
 */
	public int semaine(int jour, int mois, int annee){
		int nbJour=0;
		int semaine=0;
		
		try{
			for(int i=1;i<mois;i++)	nbJour+=jourMois(i,annee);
			nbJour+=jour;
			if (jourSemaine(1,1,annee)==0) semaine=(nbJour/7)+1;
			else {
				nbJour=nbJour-(8-jourSemaine(1,1,annee));
				if (nbJour<0) semaine=0;
				else semaine=(nbJour/7)+1;
			}
			return semaine;
		}
		
		catch (DATEException e){
			return 0;
		}
	}
	
/** Calcul le num&eacute;ro de semaine de l'instance.
 * @return la valeur de la semaine.
 */
	public int semaine(){
		return semaine(m_Jour,m_Mois,m_Annee);
	}
	
/** Permet d'incr&eacute;menter de 1 mois la date.
 */
	public void incMois(){
		if (m_Mois==12){
			m_Mois=1;
			m_Annee++;
		}
		else m_Mois++;
	}
		
/** Permet de d&eacute;cr&eacute;menter de 1 mois la date.
 */
	public void decMois(){
		if (m_Mois==1){
			m_Mois=12;
			m_Annee--;
		}
		else m_Mois--;
	}
/** Permet de d&eacute;caler la date d'un certain nombre de jour.
 * @param j le nombre de jour de d&eacute;calage.
 */
	public void decalJour(int j){
		try{
			if (j>0){
				for (int i=0;i<j;i++){
					if (m_Jour==jourMois(m_Mois,m_Annee)){
						m_Jour=1;
						incMois();
					}
					else m_Jour++;
				}
			}
			else{
				for (int i=0;i<(-j);i++){
					if (m_Jour==1){
						decMois();
						m_Jour=jourMois(m_Mois,m_Annee);
					}
					else m_Jour--;
				}
			}
		}
		catch (DATEException e){}
	}

/** Permet de d&eacute;caler la date d'un certain nombre d'annes.
 * @param a le nombre d'anne de d&eacute;calage.
 */
	public void decalAnnee(int a){
		if((m_Mois==2)&&(m_Jour==29)&&(bissextile(m_Annee+a)==false)){
			m_Jour=28;
		}
		m_Annee+=a;
	}

/** Permet de retourner la dernire semaine de l'anne.
 */
	public int semaineMax(){
		return semaine(31,12,m_Annee);
	}

/** Permet de retourner la premire semaine de l'anne (0 ou 1).
 */
	public int semaineMin(){
		return semaine(1,1,m_Annee);
	}

/** Retourne la date du lundi de la semaine spcifie.
 * @param s le numro de la semaine.
 */
	public DATE getDateLundi(int s){
		int a = getAnnee();
		try{
			DATE d = new DATE(1,1,a);
			while(jourSemaine(d.getJour(),d.getMois(),d.getAnnee())!=0){
				d.decalJour(1);
			}
			d.decalJour(7*(s-1));
			return d;
		}
		catch (DATEException e){
			return this;
		}
	}

}

	