package SmartAgenda.Moteur;

import java.io.*;
import java.net.*;
import java.util.*;
import java.lang.*;

import Outils.*;
import SmartAgenda.Net.*;

/** Classe permettant la gestion des utilisateurs.*/
public class UTILISATEUR {
	private String m_Nom="";
	private String m_Prenom="";
	
	/* Identifiant unique pour Chacun des utilisateurs. */
	private String m_Identifiant=""; /* Gestion interne */
	private String m_Login="";
	private String m_PassWord="";
	
	/* Password a donner lors du login. Le password sera genere a partir du nom de l'utilisateur. Il doit etre unique. Au cas au il existe des noms identiques, c"est a l'administrateur de determiner un nouveau password. */
	private String m_Password;
	private FICHE m_Adresse=null;
	
/** Cr&eacute;ation de l'instance de la classe.
 */
	public UTILISATEUR() {
		m_Adresse = new FICHE();
	}

/** Cr&eacute;ation de l'instance de la classe.
 * @param id identifiant de l'utilisateur.
 * @param nom nom de l'utilisateur.
 * @param prenom prenom de l'utilisateur.
 * @param login login de l'utilisateur.
 * @param pswd password de l'utilisateur.
 */
	public UTILISATEUR(String id, String nom, String prenom, String login, String pswd) {
		m_Nom=nom;
		m_Identifiant=id;
		m_Prenom=prenom;
		m_Login=login;
		m_PassWord=pswd;
		m_Adresse = new FICHE();
	}
/** Constructeur utilis&eacute; lors du chargement du fichier agd.
 * @param s Chaine de caract&egrave;res contenant toutes les donn&eacute;es n&eacute;cessaire &agrave; la cr&eacute;ation de l'instance.
 */
	public UTILISATEUR(String s) {
		String delim = "<=>";
		String[] buf;
		String[] adresse;
		int sAdresse;
		m_Adresse = new FICHE();
		StringTokenizer st = new StringTokenizer(s,delim);
		int limite = st.countTokens();
		buf=new String[limite];
		for(int i=0;i<(limite);i++)buf[i]=st.nextToken();
		if (buf[0].compareTo("UTIL")==0){
			for(int i=1;i<(limite);i++){
				if (buf[i].compareTo("ID")==0){
					m_Identifiant=buf[i+1];
					i++;
				}
				else if (buf[i].compareTo("NOM")==0){
					m_Nom=buf[i+1];
					i++;
				}
				else if (buf[i].compareTo("PRENOM")==0){
					m_Prenom=buf[i+1];
					i++;
				}
				else if (buf[i].compareTo("LOGIN")==0){
					m_Login=buf[i+1];
					i++;
				}
				else if (buf[i].compareTo("PASSWORD")==0){
					m_PassWord=buf[i+1];
					i++;
				}
				else if (buf[i].compareTo("ADRESSE")==0){
					i++;
					adresse=new String[limite-i];
					for(sAdresse=0;sAdresse<limite-i;sAdresse++){
						adresse[sAdresse]=buf[i+sAdresse];
					}
					m_Adresse=new FICHE(adresse,sAdresse);
					i=limite;
				}
			}
		}
		else Global.msgErreur("UTILISATEUR.constructeur : not an UTILISATEUR file",false);
	}

/** Constructeur utilis&eacute; lors du chargement du fichier xml.
 * @param s Chaine de caract&egrave;res contenant toutes les donn&eacute;es n&eacute;cessaire &agrave; la cr&eacute;ation de l'instance.
 * @param b vaut "true" pour XML ("false"  ajouter pour AGD et donc remplacer le constructeur ci-dessus)
 */
//	public UTILISATEUR(String s,boolean b) {				/*ATTENTION ne fonctionne pas*/
//		String delim = "<=>";
//		String[] buf;
//		String[] adresse;
//		int sAdresse;
//		m_Adresse = new FICHE();
//		StringTokenizer st = new StringTokenizer(s,delim);
//		int limite = st.countTokens();
//		buf=new String[limite];
//		for(int i=0;i<(limite);i++)buf[i]=st.nextToken();
//		if (buf[0].compareTo("UTIL")==0){
//			for(int i=1;i<(limite);i++){
//				if (buf[i].compareTo("ID")==0){
//					m_Identifiant=buf[i+1];
//					i++;i++;
//				}
//				else if (buf[i].compareTo("NOM")==0){
//					m_Nom=buf[i+1];
//					i++;i++;
//				}
//				else if (buf[i].compareTo("PRENOM")==0){
//					m_Prenom=buf[i+1];
//					i++;i++;
//				}
//				else if (buf[i].compareTo("LOGIN")==0){
//					m_Login=buf[i+1];
//					i++;i++;
//				}
//				else if (buf[i].compareTo("PASSWORD")==0){
//					m_PassWord=buf[i+1];
//					i++;i++;
//				}
//				else if (buf[i].compareTo("ADRESSE")==0){
//					i++;
//					adresse=new String[limite-i];
//					for(sAdresse=0;sAdresse<limite-i;sAdresse++){
//						adresse[sAdresse]=buf[i+sAdresse];
//					}
//					m_Adresse=new FICHE(adresse,sAdresse);
//					i=limite;
//				}
//			}
//		}
//		else Global.msgErreur("UTILISATEUR.constructeur : not an UTILISATEUR file",false);
//	}

/** Cr&eacute;ation de l'instance.
 * @param liste liste des donn&eacute;es de l'utilisateur: [0]:identifiant,[1]:nom,[2]:prenom,[3]:login,[4]:password.
 * @param lPostal liste des adresses postales.
 * @param lEmail liste des emails.
 * @param lPortable liste des num&eacute;ros de portables.
 */
	public UTILISATEUR(String[] liste,Vector lPostal,Vector lEmail,Vector lPortable) {
/* Ordre des elements [i] en fonction des champs de l'interface
   [0] : identifiant, [1] : nom, [2] : prenom, [3] : login, [4] : password*/
		String[] postal = new String[6];
		String[] autre = new String[2];
		m_Identifiant=liste[0];
		m_Nom=liste[1];
		m_Prenom=liste[2];
		m_Login=liste[3];
		m_PassWord=liste[4];
		m_Adresse = new FICHE(lPostal,lEmail,lPortable);
	}

/** Affichage texte de l'instance.
 * @param adm variable permettant de savoir si l'utilisateur de l'agenda est un administrateur ou pas.
 */
	public void affiche(boolean adm){
		if (adm) System.out.print("<"+m_Identifiant+"> ");
		System.out.print(m_Nom+" "+m_Prenom);
		if (adm) System.out.println(" "+m_Login+" "+m_PassWord);
		else System.out.println();
		try{
			m_Adresse.affiche(adm);
		}
		catch(NullPointerException e){}
		System.out.println("");
	}

/** utilis&eacute; par l'affichage texte d'une plage horaire. */
	public void afficheLight(){
		System.out.println("<"+m_Identifiant+"> "+m_Nom+" "+m_Prenom);
		}
		
/** Sauvegarde de l'utilisateur dans un fichier.
 * @param fic fichier de sauvegarde.
 * @param xml si vrai alors suivre la DTD.
 */
	public void save(FileWriter fic,boolean xml){
		FileInputStream ficIniInput;
		FileWriter ficIniOutput;
		String fichier=AGENDA.fAgenda()+"."+AGENDA.INI;
		byte[] data;
		Integer id = new Integer(m_Identifiant);
		String s;
		String delim = new String(Global.CRLF);
		StringTokenizer st;
		int limite;
		try{
			fic.write("<ID>"+Global.nonVide(m_Identifiant,"0"));
			if (xml) fic.write("</ID>"+Global.CRLF);
			if (m_Nom.length()>0){
				fic.write("<NOM>"+m_Nom);
				if (xml) fic.write("</NOM>"+Global.CRLF);
			}
			if (m_Prenom.length()>0){
				fic.write("<PRENOM>"+m_Prenom);
				if (xml) fic.write("</PRENOM>"+Global.CRLF);
			}
			if (m_Login.length()>0){
				fic.write("<LOGIN>"+m_Login);
				if (xml) fic.write("</LOGIN>"+Global.CRLF);
			}
			if (m_PassWord.length()>0){
				fic.write("<PASSWORD>"+m_PassWord);
				if (xml) fic.write("</PASSWORD>"+Global.CRLF);
			}
			try{
				m_Adresse.save(fic,xml);
			}
			catch (NullPointerException e){}
		}
		catch (IOException e)
		{
			Global.msgErreur("UTILISATEUR.insert : "+e.getMessage(),false);
		}
		if (!xml){
			try{
				ficIniInput = new FileInputStream(fichier);
				data = new byte[ficIniInput.available()];
				ficIniInput.read(data);
				ficIniInput.close();
				
				s = new String(data);
				st = new StringTokenizer(s,delim);
				limite = st.countTokens();
				String[] contenu = new String[limite];
				
				for(int i=0;i<limite;i++){
					try{
						contenu[i]=st.nextToken();
					}
					catch (NoSuchElementException e){
						contenu[i]="";
					}
				}
				ficIniOutput = new FileWriter(fichier,false);
				for(int i=0;i<(limite-1);i++){
					if (contenu[i].compareTo("[LAST ID]")==0){
						contenu[i+1]=""+AGENDA.getLastId();
						ficIniOutput.write(contenu[i]+Global.CRLF);
						ficIniOutput.write(contenu[i+1]+Global.CRLF);
					}
					else ficIniOutput.write(contenu[i]+Global.CRLF);
				}
				ficIniOutput.close();
			}
			catch (IOException e){
				Global.msgErreur("UTILISATEUR.insert : "+e.getMessage(),false);
			}
		}
	}
	
/** Permet de r&eacute;cup&eacute;rer l'identifiant de l'utilisateur.
 * @return l'identifiant de l'utilisateur.
 */
	public String identifiant(){
		return m_Identifiant;
	}
	
/** Recherche d'un utilisteur a partir de son ID.
 * @param liste liste des utilisateurs.
 * @param id id de l'utilisateur recherch&eacute;.
 * @return l'utilisateur recherch&eacute; s'il existe, null sinon.
 */
	public static UTILISATEUR rechercheUtilisateur(Vector liste, String id){
		UTILISATEUR utilisateur;
		for(int i=0;i<liste.size();i++){
			utilisateur=(UTILISATEUR)liste.elementAt(i);
			if ((utilisateur.identifiant()).compareTo(id)==0) return utilisateur;
		}
		return null;
	}

/** Ajout d'un adresse postale.
 * @param postal l'adresse postale &agrave; ajouter.
 */
	public void addAdresse (POSTAL postal){
		m_Adresse.addAdresse(postal);
	}

/** Ajout d'un adresse email.
 * @param email l'adresse email &agrave; ajouter.
 */
	public void addEmail (E_MAIL email){
		m_Adresse.addEmail(email);
	}

/** Ajout d'un num&eacute;ro de portable.
 * @param portable le num&eacute;ro de portable &agrave; ajouter.
 */
	public void addPortable (PORTABLE portable){
		m_Adresse.addPortable(portable);
	}
	
/** verification des droits de connection.
 * @param login login de la personne voulant se connecter.
 * @param password mot de passe de la personne voulant se connecter (non utilis&eacute; pour le moment).
 */
	public boolean verifAutorisation (String login, String password){
		if ((login.compareTo(m_Login)==0)&&(password.compareTo(m_PassWord)==0)) return true;
		else return false;
	}
	
/** verification des droits de connection.
 * @param login login de la personne voulant se connecter.
 */
	public boolean verifAutorisation (String login){
		if (login.compareTo(m_Login)==0) return true;
		else return false;
	}
	
/** Permet de r&eacute;cup&eacute;rer l'identifiant de l'utilisateur.
 * @return l'identifiant de l'utilisateur.
 */
	public String getId(){
		return m_Identifiant;
	}
	
/** Permet de r&eacute;cup&eacute;rer le nom de l'utilisateur.
 * @return le nom de l'utilisateur.
 */
	public String getNom(){
		return m_Nom;
	}
	
/** Permet de r&eacute;cup&eacute;rer le pr&eacute;nom de l'utilisateur.
 * @return le pr&eacute;nom de l'utilisateur.
 */
	public String getPrenom(){
		return m_Prenom;
	}
	
/** Permet de r&eacute;cup&eacute;rer le login de l'utilisateur.
 * @return le login de l'utilisateur.
 */
	public String getLogin(){
		return m_Login;
	}

/** Permet de r&eacute;cup&eacute;rer le password de l'utilisateur.
 * @return le password de l'utilisateur.
 */
	public String getPassWord(){
		return m_PassWord;
	}

/** Permet de r&eacute;cup&eacute;rer les adresses de l'utilisateur.
 * @return les adresses de l'utilisateur.
 */
	public FICHE getAdresses(){
		return m_Adresse;
	}
	
/** Compare l'ordre de grandeur d'un utilisateur avec l'instance. <br> Apr&egrave;s avoir tester l'ordre lexicographique du nom, la m&eacute;thode teste l'ordre lexicographique des pr&eacute;noms.
 * @param u l'utilisateur &agrave; comparer.
 */
	public boolean plusGrand(UTILISATEUR u){
		if(m_Nom.toUpperCase().compareTo(u.getNom().toUpperCase())>0) return true;
		else if (m_Nom.toUpperCase().compareTo(u.getNom().toUpperCase())<0) return false;
		else if (m_Prenom.toUpperCase().compareTo(u.getPrenom().toUpperCase())<0) return false;
		else if (m_Prenom.toUpperCase().compareTo(u.getPrenom().toUpperCase())>0) return true;
		else return false;
	}

/** Permet de supprimer une adresse postale.
 * @param i  l'index de l'adresse &agrave; supprimer.
 * @return vrai si la suppression s'est bien effectu&eacute;e.
 */
	public boolean removeAdresse(int i){
		return m_Adresse.removePostal(i);
	}
	
/** Permet de supprimer une adresse email.
 * @param i  l'index de l'email &agrave; supprimer.
 * @return vrai si la suppression s'est bien effectu&eacute;e.
 */
	public boolean removeEmail(int i){
		return m_Adresse.removeEmail(i);
	}
	
/** Permet de supprimer un num&eacute;ro de portable.
 * @param i  l'index du num&eacute;ro de portable &agrave; supprimer.
 * @return vrai si la suppression s'est bien effectu&eacute;e.
 */
	public boolean removePortable(int i){
		return m_Adresse.removePortable(i);
	}
	
/** Permet de modifier les caract&eacute;ristiques de l'utilisateur.
 * @param nom le nouveau nom.
 * @param prenom le nouveau prenom.
 * @param login le nouveau login.
 * @param pswd le nouveau password.
 */
 	public void modif (String nom, String prenom, String login, String pswd){
		m_Nom=nom;
		m_Prenom=prenom;
		m_Login=login;
		m_PassWord=pswd;
	}
	
/** Permet de modifier l'identifiant de l'utilisateur.
 * @param i le nouvel identifiant.
 */
	public void setId (int i){
		m_Identifiant=""+i;
	}
	
/** Permet de sauvegarder la base de donn&eacute;es sur un serveur.
 * @param first variable d&eacute;terminant s'il s'agit du premier paquet envoy&eacute; au serveur ou pas.
 * @param last variable d&eacute;terminant s'il s'agit du dernier paquet envoy&eacute; au serveur ou pas.
 * @param url url du serveur. 
 * @return vrai si le transfert s'est bien effectu&eacute;, faux sinon.
 */
	public boolean saveDistant(boolean first,boolean last,String url){
		String val;
		String msg;
		boolean resultat=false;
		
		if (m_Adresse==null){
			if(first && !last) val="new";
			else if(first && last) val="only";
			else if(!first && last) val="close";
			else val="app";
		}
		else if (m_Adresse.vide()){
			if(first && !last) val="new";
			else if(first && last) val="only";
			else if(!first && last) val="close";
			else val="app";
		}
		else{
			if (first) val="new";
			else val="app";
		}
		
		msg="val="+DataURLAccess.urlEncode(val)+"&id="+DataURLAccess.urlEncode(m_Identifiant)+"&nom="+DataURLAccess.urlEncode(m_Nom)+"&prenom="+DataURLAccess.urlEncode(m_Prenom)+"&login="+DataURLAccess.urlEncode(m_Login);
		resultat= DataURLAccess.writeData(url,msg);
		
		if (resultat) return m_Adresse.saveDistant(first,last,url);
		else return false;
	}
}
